/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 27/08/2015
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Customattribute;
import java.util.Date;
import javax.persistence.EntityManager;
import javax.persistence.Query;

/**
 * CustomattributeMgr
 * 
 */
public class CustomattributeMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public CustomattributeMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the customattribute with the specified uuid.
     *
     * @param uuid the unique id of the customattribute
     * @return the customattribute with the specified uuid
     * @throws Exception
     */
    public Customattribute getCustomattributeByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Customattribute.findByUuid");
        q.setParameter("uuid", uuid);
        return (Customattribute) q.getSingleResult();
    }

    /**
     * Returns the customattribute with the specified name.
     *
     * @param name the name of the customattribute
     * @return the customattribute with the specified name
     * @throws Exception
     */
    public Customattribute getCustomattributeByName(String name) throws Exception {
        //Checking parameters
        if (name == null) {
            throw new Exception("The name passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Customattribute.findByName");
        q.setParameter("name", name);
        return (Customattribute) q.getSingleResult();
    }

    /**
     * Insert the specified customattributes in the database. Returns true if
     * the operation succeed.
     *
     * @param ca the customattribute object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertCustomattribute(Customattribute ca) throws Exception {
        //Checking parameters
        if (ca == null) {
            throw new Exception("The Customattribute passed by parameter is null.");
        }
        
        if (ca.getDatecreation()== null) {
            ca.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ca);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(ca);

        return true;
    }

    /**
     * Update the customattribute specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param ca the updated customattribute (this method update the
     * customattribute on the db with the customattribute passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateCustomattribute(Customattribute ca) throws Exception {
        //Checking parameters
        if (ca == null) {
            throw new Exception("The Customattribute passed by parameter is null.");
        }

        if (ca.getDatelastupdate() == null) {
            ca.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ca, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(ca);

        return true;
    }

    /**
     * Delete from the database the customattribute specified by parameter.
     * Returns true if the operation succeed.
     *
     * @param ca the customattribute do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteCustomattribute(Customattribute ca) throws Exception {
        //Checking parameters
        if (ca == null) {
            throw new Exception("The Customattribute passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(ca);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Customattribute ca) throws Exception {
        if (ca.getUuid() == null || ca.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ca.getName() == null) {
            throw new Exception("Name field not available.");
        }

        if (ca.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Customattribute ca, boolean isUpdate) throws Exception {
        if (ca.getUuid() == null || ca.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ca.getName() == null) {
            throw new Exception("Name field not available.");
        }

        if (ca.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }
        if (isUpdate && ca.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }
}
