/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 27/08/2015
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Customattributecode;
import javax.persistence.EntityManager;
import javax.persistence.Query;

/**
 * CustomattributecodeMgr
 * 
 */
public class CustomattributecodeMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public CustomattributecodeMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the customattributecode with the specified uuid.
     *
     * @param uuid the unique id of the customattributecode
     * @return the customattributecode with the specified uuid
     * @throws Exception
     */
    public Customattributecode getCustomattributecodeByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Customattributecode.findByUuid");
        q.setParameter("uuid", uuid);
        return (Customattributecode) q.getSingleResult();
    }

    /**
     * Insert the specified customattributecodes in the database. Returns true
     * if the operation succeed.
     *
     * @param ca the customattributecode object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertCustomattributecode(Customattributecode ca) throws Exception {
        //Checking parameters
        if (ca == null) {
            throw new Exception("The Customattributecode passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(ca);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(ca);

        return true;
    }

    /**
     * Update the customattributecode specified by parameter. Returns true if
     * the operation succeed.
     *
     * @param ca the updated customattributecode (this method update the
     * customattributecode on the db with the customattributecode passed by
     * parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateCustomattributecode(Customattributecode ca) throws Exception {
        //Checking parameters
        if (ca == null) {
            throw new Exception("The Customattributecode passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(ca);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(ca);

        return true;
    }

    /**
     * Delete from the database the customattributecode specified by parameter.
     * Returns true if the operation succeed.
     *
     * @param ca the customattributecode do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteCustomattributecode(Customattributecode ca) throws Exception {
        //Checking parameters
        if (ca == null) {
            throw new Exception("The Customattributecode passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(ca);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Customattributecode ca) throws Exception {
        if (ca.getUuid() == null || ca.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ca.getCustomattribute() == null) {
            throw new Exception("Customattribute field not available.");
        }

        if (ca.getValue() == null) {
            throw new Exception("Value field not available.");
        }
        return true;
    }

}
