/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 27/08/2015
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.constants.Constants;
import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Procedurestatus;
import eu.europa.ec.re3gistrycommon.model.Users;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class DataprocedureMgr {

    private final EntityManager em;
    private static final Object sync = new Object();

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public DataprocedureMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns all dataimport completed between the 2 date parameters
     *
     * @param lastCompletedStaticizerDatestart
     * @param inprogressStaticizerDatestart
     * @return all the dataimport completed
     * @throws Exception
     */
    public List<Dataprocedure> findAllBetweenDatesOrderAscByDatecompleted(Date lastCompletedStaticizerDatestart, Date inprogressStaticizerDatestart) throws Exception {
        if (lastCompletedStaticizerDatestart == null && inprogressStaticizerDatestart == null) {
            throw new Exception("The lastCompletedStaticizerDatestart or inprogressStaticizerDatestart passed by parameter is null.");
        }

        Query q = this.em.createNamedQuery("Dataprocedure.findAllBetweenDatesOrderAscByDatecompleted");
        q.setParameter("dateinit", lastCompletedStaticizerDatestart);
        q.setParameter("datefinal", inprogressStaticizerDatestart);

        return (List<Dataprocedure>) q.getResultList();
    }

    /**
     * @return a list of all the Dataprocedure
     *
     * @throws Exception
     */
    public List<Dataprocedure> findAll() throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Dataprocedure.findAll");
        return (List<Dataprocedure>) q.getResultList();
    }

    /**
     * @return a list of Dataprocedure completed ordered ASC
     *
     * @throws Exception
     */
    public List<Dataprocedure> findAllOrderAscByDatecompleted() throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Dataprocedure.findAllOrderAscByDatecompleted");
        return (List<Dataprocedure>) q.getResultList();
    }

    /**
     * Returns the cdataimport with the specified uuid.
     *
     * @param uuid the unique id of the cdataimport
     * @return the cdataimport with the specified uuid
     * @throws Exception
     */
    public Dataprocedure getDataprocedureByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Dataprocedure.findByUuid");
        q.setParameter("uuid", uuid);
        return (Dataprocedure) q.getSingleResult();
    }

    /**
     * Insert the specified cdataimports in the database. Returns true if the
     * operation succeed.
     *
     * @param di the cdataimport object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertDataprocedure(Dataprocedure di) throws Exception {
        //Checking parameters
        if (di == null) {
            throw new Exception("@@ The Dataprocedure passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(di);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("@@ The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(di);

        return true;
    }

    /**
     * Update the cdataimport specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param di the updated cdataimport (this method update the cdataimport on
     * the db with the cdataimport passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateDataprocedure(Dataprocedure di) throws Exception {
        //Checking parameters
        if (di == null) {
            throw new Exception("@@ The Dataprocedure passed by parameter is null.");
        }

        //Checking required fields
        this.checkMandatoryFields(di);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("@@ The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(di);

        return true;
    }

    /**
     * Delete from the database the cdataimport specified by parameter. Returns
     * true if the operation succeed.
     *
     * @param di the cdataimport do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteDataprocedure(Dataprocedure di) throws Exception {
        //Checking parameters
        if (di == null) {
            throw new Exception("@@ The Dataprocedure passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("@@ The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(di);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Dataprocedure di) throws Exception {
        if (di.getUuid() == null || di.getUuid().trim().equals("")) {
            throw new Exception("@@ Uuid field not available.");
        }

        if (di.getFolder() == null) {
            throw new Exception("@@ Folder field not available.");
        }

        if (di.getDatestarted() == null) {
            throw new Exception("@@ Datestarted field not available.");
        }

        return true;
    }

    /*
     * Save the current procedure to the list of executed procedure
     */
    public Dataprocedure storeDataProcedure(EntityManager em, String procedureDescription, String statusUUID, Users user, String folder, boolean ingoreWarning) throws Exception {
        synchronized (sync) {
            //Saving this procedure to the procedute table
            DataprocedureMgr dataprocedureMgr = new DataprocedureMgr(em);
            Dataprocedure tmpDataprocedure = new Dataprocedure();
            ProcedurestatusMgr procedurestatusMgr = new ProcedurestatusMgr(em);
            Procedurestatus inprogress = procedurestatusMgr.getStatusByUuid(statusUUID);

            String dataprocedureUuid = Dataprocedure.createUuid(System.currentTimeMillis());

            tmpDataprocedure.setUuid(dataprocedureUuid);
            tmpDataprocedure.setUsers(user);
            tmpDataprocedure.setFolder(folder);
            tmpDataprocedure.setComment(procedureDescription);
            tmpDataprocedure.setDatestarted(new Date());
            tmpDataprocedure.setProcedurestatus(inprogress);
            tmpDataprocedure.setIgnorewarning(ingoreWarning);

            dataprocedureMgr.insertDataprocedure(tmpDataprocedure);

            return tmpDataprocedure;
        }
    }

    public void updateDataProcedureStatus(EntityManager em, Dataprocedure dataprocedure, String statusUUID) throws Exception {
        synchronized (sync) {
            //Saving this procedure to the procedute table
            DataprocedureMgr dataprocedureMgr = new DataprocedureMgr(em);

            ProcedurestatusMgr procedurestatusMgr = new ProcedurestatusMgr(em);
            Procedurestatus status = procedurestatusMgr.getStatusByUuid(statusUUID);

            dataprocedure.setProcedurestatus(status);
            dataprocedure.setDatecompleted(new Date());

            dataprocedureMgr.updateDataprocedure(dataprocedure);
        }
    }

    public boolean isProcedureInProgress(EntityManager em) {
        synchronized (sync) {
            //Checking if there is another procedure running
            DataprocedureMgr dataProcedureMgr = new DataprocedureMgr(em);
            boolean otherProcedure = false;
            try {
                ProcedurestatusMgr procedurestatusMgr = new ProcedurestatusMgr(em);
                Procedurestatus storingToDb = procedurestatusMgr.getStatusByUuid(Constants.ID_SAVING_TO_DB_STATUS);
                Procedurestatus writingFiles = procedurestatusMgr.getStatusByUuid(Constants.ID_WRITING_FILES_STATUS);
                Procedurestatus checkingFiles = procedurestatusMgr.getStatusByUuid(Constants.ID_CHECKINGFILES_STATUS);

                if (dataProcedureMgr.findByProcedurestatus(storingToDb).size() > 0 || dataProcedureMgr.findByProcedurestatus(writingFiles).size() > 0 || dataProcedureMgr.findByProcedurestatus(checkingFiles).size() > 0) {
                    otherProcedure = true;
                }
            } catch (Exception e){
            }
            return otherProcedure;
        }
    }

    /**
     * @param procedurestatus
     * @return a list of Dataprocedure by the status parameter
     *
     * @throws Exception
     */
    public List<Dataprocedure> findByProcedurestatus(Procedurestatus procedurestatus) throws Exception {
        synchronized (sync) {
            //Preparing query
            Query q = this.em.createNamedQuery("Dataprocedure.findByProcedurestatus");
            q.setParameter("procedurestatus", procedurestatus);
            return (List<Dataprocedure>) q.getResultList();
        }
    }
}
