/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 27/08/2015
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrydata.managers;

import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import eu.europa.ec.re3gistrycommon.managers.CustomattributeMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemclassMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemclasscustomattributeMgr;
import eu.europa.ec.re3gistrycommon.model.Customattribute;
import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Itemclasscustomattribute;
import eu.europa.ec.re3gistrydata.beans.Field;
import eu.europa.ec.re3gistrycommon.utils.Report;
import eu.europa.ec.re3gistrydata.constants.Constants;
import eu.europa.ec.re3gistrydata.managers.actions.AdditionMgr;
import eu.europa.ec.re3gistrydata.managers.actions.ClarificationMgr;
import eu.europa.ec.re3gistrydata.managers.actions.InvalidationMgr;
import eu.europa.ec.re3gistrydata.managers.actions.RetirementMgr;
import eu.europa.ec.re3gistrydata.managers.actions.SupersessionMgr;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;

public class StoreMgr {

    //static variables
    private static final Logger log = Logger.getLogger(Constants.MODULE_NAME);

    //Common variables
    private final ReportMgr reportMgr;
    private final DataMgr dataMgr;

    //Custom attributes list
    private final List<Field> customAttributeDescriptors;

    //Managers
    private AdditionMgr additionMgr;
    private RetirementMgr retirementMgr;
    private InvalidationMgr invalidationMgr;
    private SupersessionMgr supersessionMgr;
    private ClarificationMgr clarificationMgr;

    //Localization
    private final ResourceBundle localizationres;

    /**
     * Class initializer
     *
     * @param reportMgr
     * @param dataMgr
     * @throws Exception
     */
    public StoreMgr(ReportMgr reportMgr, DataMgr dataMgr) throws Exception {
        log.info("# Initializing the store manager ...");

        this.reportMgr = reportMgr;
        this.dataMgr = dataMgr;

        //Initializing list
        customAttributeDescriptors = dataMgr.getCustomAttributeDescriptors();

        //Initializing the system localization
        HttpSession s = dataMgr.getRequest().getSession();
        String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
        if (currentLanguage == null) {
            currentLanguage = dataMgr.getRequest().getLocale().toString();
        }
        this.localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));

        log.info("# Store manager initialized.");
    }

    /**
     * Start the data store procedure
     *
     * @param procedureDescription
     * @param em
     * @param dataprocedure
     * @throws Exception
     */
    public void storeData(String procedureDescription, EntityManager em, Dataprocedure dataprocedure) throws Exception {
        log.info("# Starting data store.");

        //Beginning transaction
        em.getTransaction().begin();

        try {
            //Initializing managers
            additionMgr = new AdditionMgr(em, reportMgr, dataMgr, dataprocedure);
            retirementMgr = new RetirementMgr(em, reportMgr, dataMgr, dataprocedure);
            invalidationMgr = new InvalidationMgr(em, reportMgr, dataMgr, dataprocedure);
            supersessionMgr = new SupersessionMgr(em, reportMgr, dataMgr, dataprocedure);
            clarificationMgr = new ClarificationMgr(em, reportMgr, dataMgr, dataprocedure);

            //Create the new custom attributes if needed
            this.checkCustomAttribute(customAttributeDescriptors, em);

            //Storing the additions
            additionMgr.storeAdditions();

            //Process retirements
            retirementMgr.processRetirements();

            //Process invalidations
            invalidationMgr.processInvalidations();

            //Process supersession
            supersessionMgr.processSupersessions();

            //Process clarifications
            clarificationMgr.storeClarifications();
        } catch (Exception e) {
            //In case of errors, all the operation are rollbacked
            em.getTransaction().rollback();
            throw e;
        }

        //Seraching for data store errors (to handle the "ignore warning" flag)
        int errors = 0;
        for (Report tmp : reportMgr.getReports()) {
            if (tmp.getType().equals(Constants.REPORT_TYPE_ERROR)) {
                errors++;
            }
        }

        if (errors > 0) {
            //In case of errors, all the operation are rollbacked
            em.getTransaction().rollback();
        } else {
            // Committing opertations
            em.getTransaction().commit();
        }
    }

    /**
     * Check and eventually creates the custom attributes specified in the data
     * file
     *
     * @param customAttributeDescriptors
     * @param em
     * @throws Exception
     */
    public void checkCustomAttribute(List<Field> customAttributeDescriptors, EntityManager em) throws Exception {

        log.info("# Starting custom attributes check/insert");

        CustomattributeMgr customattributeMgr = new CustomattributeMgr(em);
        ItemclasscustomattributeMgr itmcaMgr = new ItemclasscustomattributeMgr(em);
        ItemclassMgr itemclassMgr = new ItemclassMgr(em);

        for (Field customAttributeDescriptor : customAttributeDescriptors) {

            String itemclassString = customAttributeDescriptor.getItemClass();

            //Getting itemclass
            Itemclass itemClass = itemclassMgr.getItemclassByUriname(itemclassString);
            if (itemClass == null) {
                throw new Exception(this.localizationres.getString("error.storemgr.cacheck.noitemclass").replace("{0}", itemclassString));
            }
            if (itemClass.getRegister() == null) {
                throw new Exception(this.localizationres.getString("error.storemgr.cacheck.noonjfromitemclass").replace("{0}", itemclassString));
            }

            Customattribute customAttribute;
            try {
                customAttribute = customattributeMgr.getCustomattributeByName(customAttributeDescriptor.getName());
            } catch (NoResultException e) {
                //If the custom attribute is not available, the system creates it.
                customAttribute = new Customattribute();

                //Setting the properties for the new custom attribute
                String customAttributeUuid = Customattribute.createUuid(customAttributeDescriptor.getName(), itemClass.getUriname());
                customAttribute.setUuid(customAttributeUuid);
                customAttribute.setName(customAttributeDescriptor.getName());
                customAttribute.setCoded(customAttributeDescriptor.isCoded());
                customAttribute.setMultivalue(customAttributeDescriptor.isMultivalued());
                customAttribute.setDatecreation(new Date());

                //Stroing custom attribute
                customattributeMgr.insertCustomattribute(customAttribute);
                log.debug("# Storing a new custom attribute: " + customAttribute.getName());

            } catch (Exception e) {
                throw new Exception(this.localizationres.getString("error.storemgr.cacheck.getca").replace("{0}", e.getMessage()));
            }

            //Checking the custom attribute assotiation
            Itemclasscustomattribute itmca;
            try {
                itmca = itmcaMgr.getItemclasscustomattributeByItemclassCustomattribute(itemClass, customAttribute);
            } catch (NoResultException e) {
                //If the assignation is not available, the system creates it
                itmca = new Itemclasscustomattribute();
                String itmcaUuid = Itemclasscustomattribute.createUuid(customAttribute.getName(), itemClass.getUriname());
                itmca.setUuid(itmcaUuid);
                itmca.setCustomattribute(customAttribute);
                itmca.setItemclass(itemClass);
                itmca.setIsforeignkey(customAttributeDescriptor.isForeignkey());
                itmca.setRequired(customAttributeDescriptor.isRequired());

                //Storing a new custom attribute itemclass assotiation
                itmcaMgr.insertItemclasscustomattribute(itmca);
                log.debug("# Storing a new custom attribute itemclass assotiation: " + customAttribute.getName() + " - " + itemClass.getUriname());
            } catch (Exception e) {
                throw new Exception(this.localizationres.getString("error.storemgr.cacheck.caitmc").replace("{0}", e.getMessage()));
            }
        }
        log.info("# Custom attributes check/insert finished");
    }

}
