/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 27/08/2015
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrydata.managers.actions;

import eu.europa.ec.re3gistrycommon.managers.ChangelogMgr;
import eu.europa.ec.re3gistrycommon.managers.CustomattributevalueMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemclassMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemcollectionMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemparentMgr;
import eu.europa.ec.re3gistrycommon.managers.ItemsuccessorMgr;
import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.managers.StatusMgr;
import eu.europa.ec.re3gistrycommon.model.Changelog;
import eu.europa.ec.re3gistrycommon.model.Customattributevalue;
import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Itemparent;
import eu.europa.ec.re3gistrycommon.model.Itemsuccessor;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Status;
import eu.europa.ec.re3gistrydata.beans.Addition;
import eu.europa.ec.re3gistrydata.beans.Clarification;
import eu.europa.ec.re3gistrydata.beans.CustomattributeDescriptor;
import eu.europa.ec.re3gistrydata.constants.Constants;
import eu.europa.ec.re3gistrydata.exceptions.ErrorReport;
import eu.europa.ec.re3gistrydata.exceptions.WarningReport;
import eu.europa.ec.re3gistrydata.managers.DataMgr;
import eu.europa.ec.re3gistrydata.managers.PropertyMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.servlet.http.HttpSession;
import org.apache.log4j.Logger;

public class ClarificationMgr {

    //static variables
    private static final Logger log = Logger.getLogger(Constants.MODULE_NAME);

    private final Properties properties;
    private final EntityManager em;
    private final ReportMgr reportMgr;
    private final DataMgr dataMgr;
    private final Dataprocedure dataprocedure;

    private final String latestVersion;

    //Hashmaps
    private final LinkedHashMap<String, Clarification> clarifications;

    //Version only one time for multi language
    private final HashMap<String, Item> increasedVersion = new HashMap<String, Item>();

    //Performance store (in order to avoid continuous access to the db)
    private final LinkedHashMap<String, Item> itemLoaded = new LinkedHashMap<String, Item>();

    //Localization
    private final ResourceBundle localizationres;

    /**
     * Class initializer
     *
     * @param em the EntityManager instance
     * @param reportMgr the ReportMgr
     * @param dataMgr
     * @param dataprocedure
     */
    public ClarificationMgr(EntityManager em, ReportMgr reportMgr, DataMgr dataMgr, Dataprocedure dataprocedure) {
        this.em = em;
        this.reportMgr = reportMgr;
        this.dataMgr = dataMgr;
        this.dataprocedure = dataprocedure;

        //Loading module properties
        this.properties = PropertyMgr.getProperties();

        //Initializing hashmap
        this.clarifications = dataMgr.getClarification();

        this.latestVersion = properties.getProperty(Constants.KEY_LATEST_VERSION);

        HttpSession s = dataMgr.getRequest().getSession();
        String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
        if (currentLanguage == null) {
            currentLanguage = dataMgr.getRequest().getLocale().toString();
        }
        this.localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));

    }

    /**
     * Store the related clarifications in the database
     *
     * @throws Exception
     */
    public void storeClarifications() throws Exception {
        log.info("# Starting to process clarifications.");

        Itemclass itemclass = null;

        //Starting the store of each clarification
        for (String key : this.clarifications.keySet()) {
            try {
                Clarification clarification = this.clarifications.get(key);

                /*Getting itemclass and status out of the storeClarification method
                 for performance reason.*/
                //Getting itemclass
                if (clarification.getItemClass() == null || clarification.getItemClass().length() <= 0) {
                    throw new Exception(this.localizationres.getString("error.clarificationmgr.stcl.noitmc"));
                }
                if (itemclass == null || (!clarification.getItemClass().equals(itemclass.getUriname()))) {
                    itemclass = this.getItemclass(clarification.getItemClass());
                }

                //Storing the current clarification
                this.storeClarification(this.clarifications.get(key));

            } catch (Exception e) {
                //If an error occurs during the DB operation for the current 
                //additions, skips the current clarification and saves a 
                //warning message
                reportMgr.addReport(Constants.REPORT_CONTEXT_CLARIFICATIONMGR, Constants.REPORT_ACTION_PROCESSCLARIFICATION, "This item is skipped: " + key + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.error("@@ This item is skipped: " + key + ". Detail: " + e.getMessage());
            }
        }

        log.info("# Clarifications stored.");
    }

    private void storeClarification(Clarification clarification) throws Exception {

        //Initializing managers
        ItemMgr itemMgr = new ItemMgr(this.em);
        LocalizationMgr localizationMgr = new LocalizationMgr(this.em);
        CustomattributevalueMgr customattributevalueMgr = new CustomattributevalueMgr(this.em);
        Item duplicatedItem = null;
        try {
            //Creating the item UUID;
            String itemUuid = Item.createUuid(clarification.getLocalId(), this.latestVersion, clarification.getCollectionLocalId(), clarification.getItemClass());

            //Starting operation on the item
            Item item;
            if (itemLoaded.containsKey(itemUuid)) {
                item = itemLoaded.get(itemUuid);
            } else {
                try {
                    //Checing if the item to be clarified (updated) is available
                    item = itemMgr.getItemByUuid(itemUuid);
                    itemLoaded.put(itemUuid, item);
                } catch (NoResultException ex) {
                    throw new Exception(this.localizationres.getString("error.clarificationmgr.stcl.nav").replace("{0}", clarification.getLocalId()));
                }
            }

            //Getting the localization to be updated
            String localizationUuid = Localization.createUuid(item.getUuid(), clarification.getLanguage());
            Localization localization;
            try {
                localization = localizationMgr.getLocalizationByUuid(localizationUuid);
            } catch (NoResultException ex) {
                String tmp = this.localizationres.getString("error.clarificationmgr.stcl.navloc").replace("{0}", clarification.getLocalId()).replace("{1}", clarification.getLanguage());
                throw new Exception(tmp);
            }

            //***Duplicate items***//
            //Get the latest version of the current item
            int currentVersion = itemMgr.getLatestVersion(item);
            //Increase the version only one time per file per item (no increase on multiple language)
            if (!increasedVersion.containsKey(item.getUuid())) {
                currentVersion++;
                increasedVersion.put(item.getUuid(), item);
            }
            
            //Check if the item has been already duplicated (eg. by another language in the same file).
            duplicatedItem = this.checkDuplicatedItem(item, itemMgr, currentVersion);

            //If the item has not been already duplicated, duplicate it;
            if (duplicatedItem == null) {

                duplicatedItem = this.duplicateItem(item, itemMgr, currentVersion);

                /*Duplicating relations*/
                //Duplicate the parent relation
                this.duplicateItemparentRelation(item, duplicatedItem);

                //Ducplicate children relation
                this.duplicateChildren(item, duplicatedItem);

                //Duplicate collection relation
                this.duplicateItemcollectionRelation(item, duplicatedItem);

                //Duplicate the collection relation
                this.duplicateCollection(item, duplicatedItem);

                //Duplicate the successor relation
                this.duplicateSuccessor(item, duplicatedItem);

                /*Duplicating item related objects*/
                //Duplicating localization
                this.duplicateLocalization(item, duplicatedItem, localizationMgr);

                //Duplicating customattributevalues
                this.duplicateCustomattributeValues(item, duplicatedItem, customattributevalueMgr, localizationMgr);

            }

            //***Update the item***///
            //Update localization
            this.updateLocalization(clarification, localization, localizationMgr);

            //Update custom attributesvalue
            this.updateCustomattributevalue(clarification, item, localization, localizationMgr);

            //Update parent
            this.updateParent(clarification, item, itemMgr);

            this.storeClarificationChangelog(item, clarification);
            this.storeClarificationChangelog(duplicatedItem, clarification);

        } catch (Exception e) {
            if (duplicatedItem != null) {
                itemMgr.deleteItem(duplicatedItem);
            }
            throw new Exception(e);
        }
    }

    private void updateLocalization(Clarification clarification, Localization localization, LocalizationMgr localizationMgr) throws Exception {
        try {
            boolean edit = false;

            //Updating the new localization item
            if (clarification.getLabel() != null && clarification.getLabel().length() > 0) {
                localization.setLabel(clarification.getLabel());
                edit = true;
            }

            if (clarification.getDefinition() != null && clarification.getDefinition().length() > 0) {
                localization.setDefinition(clarification.getDefinition());
                edit = true;
            }
            if (clarification.getDescription() != null && clarification.getDescription().length() > 0) {
                localization.setDescription(clarification.getDescription());
                edit = true;
            }

            if (edit) {
                localization.setDatelastupdate(new Date());
                localizationMgr.updateLocalization(localization);
                log.debug("# Updating localization: " + localization.getLabel() + " - " + localization.getItem().getUriname() + " - " + localization.getLanguage().getIsocode() + " - " + localization.getItem().getItemclass().getUriname() + ".");
            }
        } catch (Exception e) {
            throw new Exception(this.localizationres.getString("error.clarificationmgr.stcl.updloc.errorloc"));
        }
    }

    private void updateParent(Clarification clarification, Item item, ItemMgr itemMgr) throws Exception {
        if (clarification.getParentLocalId() != null && clarification.getParentLocalId().length() > 0) {
            ItemparentMgr itemparentMgr = new ItemparentMgr(this.em);

            Itemparent itemparent = itemparentMgr.getLatestItemparent(item);

            String newParentUuid = Item.createUuid(clarification.getParentLocalId(), this.latestVersion, (clarification.getCollectionLocalId()!=null ? clarification.getCollectionLocalId() : null), clarification.getItemClass());

            Item parent = null;
            try {
                parent = itemMgr.getItemByUuid(newParentUuid);

                itemparent.setParent(parent);

                itemparentMgr.updateItemparent(itemparent);

            } catch (NoResultException e) {
                //error.clarificationmgr.stcl.parentnr
                log.info("Clarification manager, update parent. The new parent cannot be retrieved: " + clarification.getCollectionLocalId() + ".");
            } catch (Exception e) {
                //error.clarificationmgr.stcl.parentgterr
                log.error("Clarification manager, update parent. Generic error while getting the new parent: " + e.getMessage());
            }

        }
    }

    private void updateCustomattributevalue(Clarification clarification, Item item, Localization localization, LocalizationMgr localizationMgr) throws Exception {
        CustomattributevalueMgr customattributevalueMgr = new CustomattributevalueMgr(this.em);

        ArrayList<CustomattributeDescriptor> customattributedescriptors = clarification.getCustomAttribures();
        for (CustomattributeDescriptor customattributedescriptor : customattributedescriptors) {
            if (customattributedescriptor.getValue() != null && customattributedescriptor.getValue().length() > 0) {

                String customAttributeUuid = Customattributevalue.createUuid(customattributedescriptor.getName(), item.getUuid(), item.getItemclass().getUriname());
                Customattributevalue customattributevalue = customattributevalueMgr.getCustomattributevalueByUuid(customAttributeUuid);

                Localization cavloc = localizationMgr.findByCustomattributevalueAndLanguagecode(customattributevalue, localization.getLanguage());
                cavloc.setLabel(customattributedescriptor.getValue());
                localizationMgr.updateLocalization(cavloc);
            }
        }
    }

    private Itemclass getItemclass(String uriname) throws Exception {
        ItemclassMgr itemclassMgr = new ItemclassMgr(em);
        Itemclass itemClass = null;
        try {
            itemClass = itemclassMgr.getItemclassByUriname(uriname);
        } catch (NoResultException e) {
            throw new Exception(this.localizationres.getString("error.clarificationmgr.stcl.noimc").replace("{0}", uriname));
        }
        if (itemClass.getRegister() == null) {
            throw new Exception(this.localizationres.getString("error.clarificationmgr.stcl.noreg").replace("{0}", uriname));
        }
        return itemClass;
    }

    public Item duplicateItem(Item item, ItemMgr itemMgr, int currentVersion) throws Exception {
        Item duplicatedItem = new Item();
        Item latestItemCollection = null;
        try {
            latestItemCollection = itemMgr.getLatestCollection(item);
        } catch (Exception e) {
            log.debug("Clarification manager, this item has no collection.");
        }
        String oldItemUuid = Item.createUuid(item.getUriname(), String.valueOf(currentVersion), (latestItemCollection != null) ? latestItemCollection.getUriname() : null, item.getItemclass().getUriname());

        //Check if the item has been already duplicated (eg. by another language in the same file).
        try {
            duplicatedItem = itemMgr.getItemByUuid(oldItemUuid);
            throw new Exception(oldItemUuid);
        } catch (Exception e) {
        }

        //Setting the new item's fields
        duplicatedItem.setUuid(oldItemUuid);
        duplicatedItem.setVersionnumber(currentVersion);

        //Copy fields from the original item
        duplicatedItem.setUriname(item.getUriname());
        duplicatedItem.setStatus(item.getStatus());
        duplicatedItem.setItemclass(item.getItemclass());
        duplicatedItem.setDatecreation(item.getDatecreation());
        duplicatedItem.setDatelastupdate(new Date());
        duplicatedItem.setDatetimeaddition(item.getDatetimeaddition());
        duplicatedItem.setDatetimeinvalidation(item.getDatetimeinvalidation());
        duplicatedItem.setDatetimeretirment(item.getDatetimeretirment());
        duplicatedItem.setDatetimesupersession(item.getDatetimesupersession());

        itemMgr.insertItem(duplicatedItem);

        return duplicatedItem;
    }

    public Item checkDuplicatedItem(Item item, ItemMgr itemMgr, int currentVersion) throws Exception {
        Item duplicatedItem;
        Item latestItemCollection = null;
        try {
            latestItemCollection = itemMgr.getLatestCollection(item);
        } catch (Exception e) {
            log.debug("Clarification manager, this item has no collection.");
        }
        String oldItemUuid = Item.createUuid(item.getUriname(), String.valueOf(currentVersion), (latestItemCollection != null) ? latestItemCollection.getUriname() : null, item.getItemclass().getUriname());

        //Check if the item has been already duplicated (eg. by another language in the same file).
        try {
            duplicatedItem = itemMgr.getItemByUuid(oldItemUuid);
            return duplicatedItem;
        } catch (Exception e) {
            return null;
        }
    }

    private void duplicateItemparentRelation(Item item, Item duplicatedItem) throws Exception {
        ItemparentMgr itemparentMgr = new ItemparentMgr(this.em);
        try {
            List<Itemparent> itemparents = itemparentMgr.findByItem(item);
            for (Itemparent tmpItp : itemparents) {
                Itemparent newItp = new Itemparent();
                String newItemparentUuid = Itemparent.createUuid(duplicatedItem.getUuid(), tmpItp.getParent().getUuid());
                newItp.setUuid(newItemparentUuid);
                newItp.setItem(duplicatedItem);
                newItp.setParent(tmpItp.getParent());
                newItp.setDatecreation(new Date());
                newItp.setDatelastupdate(new Date());
                itemparentMgr.insertItemparent(newItp);
            }
        } catch (NoResultException e) {
            log.debug("Clarification manager, this item has no parent relation.");
        }

        //removing latest item old links
        itemparentMgr.cleanOldParentRelations(item);
    }

    private void duplicateChildren(Item item, Item duplicatedItem) throws Exception {
        ItemparentMgr itemparentMgr = new ItemparentMgr(this.em);
        try {
            List<Itemparent> itemparents = itemparentMgr.findByParent(item);
            for (Itemparent tmpItp : itemparents) {
                Itemparent newItp = new Itemparent();
                String newItemparentUuid = Itemparent.createUuid(tmpItp.getItem().getUuid(), duplicatedItem.getUuid());
                newItp.setUuid(newItemparentUuid);
                newItp.setParent(duplicatedItem);
                newItp.setItem(tmpItp.getItem());
                newItp.setDatecreation(new Date());
                newItp.setDatelastupdate(new Date());
                itemparentMgr.insertItemparent(newItp);
            }
        } catch (NoResultException e) {
            log.debug("Clarification manager, this item has no children.");
        }

        //removing latest item old links
        itemparentMgr.cleanOldChildrenRelations(item);
    }

    private void duplicateItemcollectionRelation(Item item, Item duplicatedItem) throws Exception {
        ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(this.em);
        try {
            List<Itemcollection> itemcollections = itemcollectionMgr.findByItem(item);
            for (Itemcollection tmpItp : itemcollections) {
                Itemcollection newItc = new Itemcollection();
                String newItemcollectionUuid = Itemcollection.createUuid(duplicatedItem.getUuid(), tmpItp.getCollection().getUuid());
                newItc.setUuid(newItemcollectionUuid);
                newItc.setItem(duplicatedItem);
                newItc.setCollection(tmpItp.getCollection());
                newItc.setDatecreation(tmpItp.getDatecreation());
                newItc.setDatelastupdate(new Date());
                itemcollectionMgr.insertItemcollection(newItc);
            }
        } catch (NoResultException e) {
            log.debug("Clarification manager, this item has no collection relation.");
        }
    }

    private void duplicateCollection(Item item, Item duplicatedItem) throws Exception {
        ItemcollectionMgr itemcollectionMgr = new ItemcollectionMgr(this.em);
        try {
            List<Itemcollection> itemcollections = itemcollectionMgr.findByCollection(item);
            for (Itemcollection tmpItc : itemcollections) {
                Itemcollection newItc = new Itemcollection();
                String newItemcollectionUuid = Itemcollection.createUuid(tmpItc.getItem().getUuid(), duplicatedItem.getUuid());
                newItc.setUuid(newItemcollectionUuid);
                newItc.setItem(tmpItc.getItem());
                newItc.setCollection(duplicatedItem);
                newItc.setDatecreation(tmpItc.getDatecreation());
                newItc.setDatelastupdate(new Date());
                itemcollectionMgr.insertItemcollection(newItc);
            }
        } catch (NoResultException e) {
            log.debug("Clarification manager, this item has no collection.");
        }
    }

    private void duplicateSuccessor(Item item, Item duplicatedItem) throws Exception {
        ItemsuccessorMgr itemsuccessorMgr = new ItemsuccessorMgr(this.em);
        try {
            List<Itemsuccessor> itemsuccessors = itemsuccessorMgr.findByItem(item);
            for (Itemsuccessor tmpIts : itemsuccessors) {
                Itemsuccessor newIts = new Itemsuccessor();
                String newItemsuccessorUuid = Itemsuccessor.createUuid(duplicatedItem.getUuid(), tmpIts.getSuccessor().getUuid());
                newIts.setUuid(newItemsuccessorUuid);
                newIts.setItem(duplicatedItem);
                newIts.setSuccessor(tmpIts.getSuccessor());
                newIts.setDatecreation(tmpIts.getDatecreation());
                newIts.setDatelastupdate(new Date());
                itemsuccessorMgr.insertItemsuccessor(newIts);
            }
        } catch (NoResultException e) {
            log.debug("Clarification manager, this item has no successor.");
        }
    }

    private void duplicateLocalization(Item item, Item duplicatedItem, LocalizationMgr localizationMgr) throws Exception {
        List<Localization> localizations = localizationMgr.findByItem(item);
        for (Localization tmpLoc : localizations) {
            this.duplicateLocalizationElement(tmpLoc, localizationMgr, duplicatedItem);
        }
    }

    private void duplicateCustomattributeValues(Item item, Item duplicatedItem, CustomattributevalueMgr customattributevalueMgr, LocalizationMgr localizationMgr) throws Exception {
        try {
            List<Customattributevalue> customattributevalues = customattributevalueMgr.findCustomAttributeValueByItem(item);
            for (Customattributevalue tmpCav : customattributevalues) {
                Customattributevalue newCav = this.duplicateCustomattributevalueElement(tmpCav, duplicatedItem, customattributevalueMgr);

                //Find the localization related to this custom attribute value
                List<Localization> tmpLocalizations = localizationMgr.findByCustomattributevalue(tmpCav);
                for (Localization tmpLoc : tmpLocalizations) {
                    this.duplicateLocalizationElemetn(tmpLoc, localizationMgr, duplicatedItem, newCav);
                }
            }
        } catch (NoResultException e) {
            log.info("Clarification manager, this item has no customattributevalue.");
        }
    }

    private void duplicateLocalizationElement(Localization localizationToDuplicate, LocalizationMgr localizationMgr, Item duplicatedItem) throws Exception {
        Localization newLoc = new Localization();
        String newLocalizationUuid = Localization.createUuid(duplicatedItem.getUuid(), localizationToDuplicate.getLanguage().getIsocode());
        newLoc.setUuid(newLocalizationUuid);
        newLoc.setItem(duplicatedItem);
        newLoc.setLanguage(localizationToDuplicate.getLanguage());
        newLoc.setLabel(localizationToDuplicate.getLabel());
        newLoc.setDefinition(localizationToDuplicate.getDefinition());
        newLoc.setDescription(localizationToDuplicate.getDescription());
        newLoc.setDatecreation(localizationToDuplicate.getDatecreation());
        newLoc.setDatelastupdate(new Date());

        localizationMgr.insertLocalization(newLoc);
    }

    public void duplicateLocalizationElemetn(Localization localizationToDuplicate, LocalizationMgr localizationMgr, Item duplicatedItem, Customattributevalue duplicatedCav) throws Exception {
        Localization newLoc = new Localization();
        String newLocalizationUuid = Localization.createUuid(duplicatedCav.getUuid() + duplicatedItem.getUuid(), localizationToDuplicate.getLanguage().getIsocode());
        newLoc.setUuid(newLocalizationUuid);
        newLoc.setCustomattributevalue(duplicatedCav);
        newLoc.setLanguage(localizationToDuplicate.getLanguage());
        newLoc.setLabel(localizationToDuplicate.getLabel());
        newLoc.setDefinition(localizationToDuplicate.getDefinition());
        newLoc.setDescription(localizationToDuplicate.getDescription());
        newLoc.setDatecreation(localizationToDuplicate.getDatecreation());
        newLoc.setDatelastupdate(new Date());

        localizationMgr.insertLocalization(newLoc);
    }

    private Customattributevalue duplicateCustomattributevalueElement(Customattributevalue cavToDuplicate, Item duplicatedItem, CustomattributevalueMgr customattributevalueMgr) throws Exception {
        Customattributevalue newCav = new Customattributevalue();
        String customattributevalueUuid = Customattributevalue.createUuid(cavToDuplicate.getCustomattribute().getName(), duplicatedItem.getUuid(), duplicatedItem.getItemclass().getUriname());

        newCav.setUuid(customattributevalueUuid);
        newCav.setItem(duplicatedItem);
        newCav.setCustomattribute(cavToDuplicate.getCustomattribute());
        newCav.setDatecreation(cavToDuplicate.getDatecreation());
        newCav.setDatelastupdate(new Date());

        customattributevalueMgr.insertCustomattributevalue(newCav);

        return newCav;
    }

    private void storeClarificationChangelog(Item item, Clarification clarification) throws Exception {
        ChangelogMgr changelogMgr = new ChangelogMgr(this.em);

        Changelog changelog = new Changelog();
        String changelogUuid = Changelog.createUuid(item, System.currentTimeMillis());

        changelog.setUuid(changelogUuid);
        changelog.setAction(Constants.ACTION_CLARIFICATION);
        changelog.setComment(clarification.getComment());
        changelog.setDataprocedure(this.dataprocedure);
        changelog.setDatecreation(new Date());
        changelog.setItem(item);
//        changelog.setLanguagecode(languagecode);

        changelogMgr.insertChangelog(changelog);
    }

    /**
     * **Analyzer***
     */
    public void analyzeClarifications() {
        log.info("# Starting to analyze clarifications.");

        //Starting the analyzer for each addition
        for (String key : this.clarifications.keySet()) {
            Clarification clarification = null;
            try {
                clarification = this.clarifications.get(key);
                this.analyzeClarification(clarification);

            } catch (ErrorReport e) {
                //If an error occurs during the analyzer operation  
                //save a error message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_CLARIFICATIONMGR, Constants.REPORT_ACTION_ANALYZECLARIFICATION, "Item: " + ((clarification != null) ? clarification.getItemClass() + "/" + clarification.getLocalId() + ((clarification.getCollectionLocalId() != null && clarification.getCollectionLocalId().length() > 0) ? "/" + clarification.getCollectionLocalId() : "") + "/" + clarification.getLanguage() : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_ERROR, -1);
                log.error("@@ Item: " + ((clarification != null) ? clarification.getItemClass() + "/" + clarification.getLocalId() + ((clarification.getCollectionLocalId() != null && clarification.getCollectionLocalId().length() > 0) ? "/" + clarification.getCollectionLocalId() : "") + "/" + clarification.getLanguage() : key) + ". Detail: " + e.getMessage());
            } catch (WarningReport e) {
                //If an warningr occurs during the analyzer operation  
                //save a warning message
                this.reportMgr.addReport(Constants.REPORT_CONTEXT_CLARIFICATIONMGR, Constants.REPORT_ACTION_ANALYZECLARIFICATION, "Item: " + ((clarification != null) ? clarification.getItemClass() + "/" + clarification.getLocalId() + ((clarification.getCollectionLocalId() != null && clarification.getCollectionLocalId().length() > 0) ? "/" + clarification.getCollectionLocalId() : "") + "/" + clarification.getLanguage() : key) + ". Detail: " + e.getMessage(), Constants.REPORT_TYPE_WARNING, -1);
                log.warn("@@ Item: " + ((clarification != null) ? clarification.getItemClass() + "/" + clarification.getLocalId() + ((clarification.getCollectionLocalId() != null && clarification.getCollectionLocalId().length() > 0) ? "/" + clarification.getCollectionLocalId() : "") + "/" + clarification.getLanguage() : key) + ". Detail: " + e.getMessage());
            }
        }

        log.info("# Clarifications analyzed.");
    }

    private void analyzeClarification(Clarification clarification) throws ErrorReport, WarningReport {

        ItemMgr itemMgr = new ItemMgr(this.em);
        //Getting the itemclass object
        ItemclassMgr itemclassMgr = new ItemclassMgr(this.em);
        Itemclass itmc = null;
        try {
            itmc = itemclassMgr.getItemclassByUriname(clarification.getItemClass());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.clarificationmgr.ancl.noitmc"));
        }

        //Checking item existence
        this.checkAvailability(itemMgr, clarification);

        //Check parent
        this.checkParent(clarification, itemMgr);

        //Check custom attribute
        this.checkCustomAttribute(clarification, itemMgr);

    }

    private void checkAvailability(ItemMgr itemMgr, Clarification clarification) throws ErrorReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        try {
            String uuidItem = Item.createUuid(clarification.getLocalId(), this.latestVersion, clarification.getCollectionLocalId(), clarification.getItemClass());
            Item item = itemMgr.getItemByUuid(uuidItem);

        } catch (NoResultException e) {
            String hashKey = Addition.getAdditionHashKey(clarification.getLocalId(), clarification.getCollectionLocalId(), clarification.getItemClass(), clarification.getLanguage());
            if (!additions.containsKey(hashKey)) {
                throw new ErrorReport(this.localizationres.getString("error.clarificationmgr.ancl.noav"));
            }
        } catch (ErrorReport e) {
            throw new ErrorReport(e.getMessage());
        } catch (Exception e) {
            throw new ErrorReport(this.localizationres.getString("error.clarificationmgr.ancl.noex").replace("{0}", e.getMessage()));
        }
    }

    private void checkCustomAttribute(Clarification clarification, ItemMgr itemMgr) throws ErrorReport, WarningReport {
        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        //Checking custom attribute
        ArrayList<CustomattributeDescriptor> customAttributeDescriptors = clarification.getCustomAttribures();

        for (CustomattributeDescriptor customAttributeDescriptor : customAttributeDescriptors) {

            if (customAttributeDescriptor.isForeignkey() && customAttributeDescriptor.getValue() != null && customAttributeDescriptor.getValue().length() > 0) {
                //Getting the external key item
                try {
                    String itemUuid = Item.createUuid(customAttributeDescriptor.getValue(), this.latestVersion, null, customAttributeDescriptor.getName());
                    Item item = itemMgr.getItemByUuid(itemUuid);

                    //Checking if the custom attribute pointed item has a valid status
                    if (!item.getStatus().getUuid().equals(Constants.ITEMSTATUS_VALID)) {
                        String tmp = this.localizationres.getString("error.clarificationmgr.ancl.novca").replace("{0}", clarification.getLocalId()).replace("{1}", customAttributeDescriptor.getName()).replace("{2}", customAttributeDescriptor.getValue());
                        throw new WarningReport(tmp);
                    }
                } catch (NoResultException e) {
                    String tmpKey = Addition.getAdditionHashKey(customAttributeDescriptor.getValue(), "", customAttributeDescriptor.getName(), clarification.getLanguage());
                    if (additions.containsKey(tmpKey)) {
                        Addition tmpAddition = additions.get(tmpKey);
                        if (tmpAddition.getStatus() != null && !tmpAddition.getStatus().equals(Constants.ITEMSTATUS_VALID_TEXT)) {
                            String tmp = this.localizationres.getString("error.clarificationmgr.ancl.navca").replace("{0}", clarification.getLocalId()).replace("{1}", customAttributeDescriptor.getName()).replace("{2}", customAttributeDescriptor.getValue());
                            throw new ErrorReport(tmp);
                        }
                    } else {
                        throw new ErrorReport(this.localizationres.getString("error.clarificationmgr.ancl.navca1"));
                    }
                } catch (ErrorReport e) {
                    throw new ErrorReport(e.getMessage());
                } catch (WarningReport e) {
                    throw new WarningReport(e.getMessage());
                } catch (Exception e) {
                    throw new ErrorReport(this.localizationres.getString("error.clarificationmgr.ancl.clerr").replace("{0}", e.getMessage()));
                }
            }
        }
    }

    private void checkParent(Clarification clarification, ItemMgr itemMgr) throws ErrorReport, WarningReport {

        LinkedHashMap<String, Addition> additions = this.dataMgr.getAdditions();

        //Checking parent
        if (clarification.getParentLocalId() != null && clarification.getParentLocalId().length() > 0) {
            try {
                String itemUuid = Item.createUuid(clarification.getParentLocalId(), this.latestVersion, clarification.getCollectionLocalId(), clarification.getItemClass());
                Item item = itemMgr.getItemByUuid(itemUuid);

                //Checking if the parent has a valid status
                if (!item.getStatus().getUuid().equals(Constants.ITEMSTATUS_VALID)) {
                    String tmp = this.localizationres.getString("error.clarificationmgr.ancl.parentnv").replace("{0}", item.getUriname()).replace("{1}", item.getStatus().getUriname());
                    throw new WarningReport(tmp);
                }
            } catch (NoResultException e) {
                String tmpKey = Addition.getAdditionHashKey(clarification.getParentLocalId(), clarification.getCollectionLocalId(), clarification.getItemClass(), clarification.getLanguage());
                if (additions.containsKey(tmpKey)) {
                    Addition tmpAddition = additions.get(tmpKey);
                    if (tmpAddition.getStatus() != null && !tmpAddition.getStatus().equals(Constants.ITEMSTATUS_VALID_TEXT)) {
                        String tmp = this.localizationres.getString("error.clarificationmgr.ancl.parentnv").replace("{0}", clarification.getParentLocalId()).replace("{1}", tmpAddition.getStatus());
                        throw new WarningReport(tmp);
                    }
                } else {
                    throw new ErrorReport(this.localizationres.getString("error.clarificationmgr.ancl.parentnav"));
                }

            } catch (ErrorReport e) {
                throw new ErrorReport(e.getMessage());
            } catch (WarningReport e) {
                throw new WarningReport(e.getMessage());
            } catch (Exception e) {
                throw new ErrorReport(this.localizationres.getString("error.clarificationmgr.ancl.parenterr").replace("{0}", e.getMessage()));
            }
        }
    }
}
