/*
 * Copyright 2010,2014 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 27/08/2015
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer;

import eu.europa.ec.re3gistrycommon.managers.ChangelogMgr;
import eu.europa.ec.re3gistrycommon.managers.DataprocedureMgr;
import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import eu.europa.ec.re3gistrycommon.model.Changelog;
import eu.europa.ec.re3gistrycommon.model.Dataprocedure;
import eu.europa.ec.re3gistrycommon.model.Item;
import eu.europa.ec.re3gistrycommon.model.Itemclass;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Users;
import eu.europa.ec.re3gistrycommon.utils.PersistenceFactory;
import eu.europa.ec.re3gistrystaticizer.constants.Constants;
import eu.europa.ec.re3gistrystaticizer.filescreation.CreateSOLRFiles;
import eu.europa.ec.re3gistrystaticizer.filescreation.CreateStandardFiles;
import eu.europa.ec.re3gistrystaticizer.filescreation.CreateVARFiles;
import eu.europa.ec.re3gistrystaticizer.filescreation.CreateXSLCustomFiles;
import java.io.File;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

public class StaticizeFiles {

    private final EntityManager em;

    private final Dataprocedure dataprocedure;
    private final ReportMgr reportMgr;
    private final HttpServletRequest request;

    private String staticizerMainfolderName;
    private String staticizerXSLFolder;
    private ResourceBundle localizationres;
    private CreateStandardFiles createStandardFiles;

    private final String folderSeparator = System.getProperty("file.separator");

    private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(Constants.MODULE_NAME);

    public StaticizeFiles(HttpServletRequest request, Dataprocedure dataprocedure, ReportMgr reportMgr) throws Exception {
        try {
            this.request = request;
            this.dataprocedure = dataprocedure;
            this.reportMgr = reportMgr;

            this.em = PersistenceFactory.getEntityManagerFactory().createEntityManager();
        } catch (Exception ex) {
            String subject = localizationres.getString("export.mail.subject.error");
            String body = localizationres.getString("export.mail.body.error.database.connection.error").replace("{user}", dataprocedure.getUsers().getEmail());
            reportMgr.setMailSubject(subject);
            reportMgr.setMailBody(body);
            reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error during the connection to the Database. Error message: " + ex.getMessage(), subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.error("=================================Error during the connection to the Database=============================================. Error message: " + ex.getMessage());
            throw new Exception("Error during the connection to the Database. Error message: " + ex.getMessage());
        }
    }

    public void startToStaticizeFiles(Dataprocedure dataprocedure) throws Exception {
        initPropertiesAndMainStaticizerFolder();
        startStaticization();
    }

    private void initPropertiesAndMainStaticizerFolder() throws Exception {
        /**
         * Initializing the system localization
         */
        HttpSession s = this.request.getSession();
        String currentLanguage = (String) s.getAttribute(Constants.LANGUAGE_SESSION_KEY);
        if (currentLanguage == null) {
            currentLanguage = this.request.getLocale().toString();
        }
        localizationres = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.MODULE_NAME + Constants.LOCALIZATION_SUFFIX, new Locale(currentLanguage));

        Properties properties = eu.europa.ec.re3gistrycommon.utils.Configuration.getInstance().getProperties(Constants.MODULE_NAME);
        log.info("Module properties loaded");

        /**
         * initialize the main staticizer folder
         */
        staticizerMainfolderName = properties.getProperty(Constants.STATICIZER_FOLDER_PATH);

        File staticizerFile = new File(staticizerMainfolderName);
        if (!staticizerFile.exists()) {
            boolean staticizerMainfolderDir = new File(staticizerMainfolderName).mkdir();
            if (!staticizerMainfolderDir) {
                String subject = localizationres.getString("export.mail.subject.error");
                String body = localizationres.getString("export.mail.body.staticizer.folder.path").replace("{user}", dataprocedure.getUsers().getEmail());
                reportMgr.setMailSubject(subject);
                reportMgr.setMailBody(body);
                reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("The path 'staticizer.custom.folder.path' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.error("===================The path 'staticizer.custom.folder.path' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.====================");
                throw new Exception("The path 'staticizer.custom.folder.path' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.");
            }

        }

        staticizerXSLFolder = properties.getProperty(Constants.STATICIZER_FORMATS_PATH);
        boolean staticizerXSLDir = new File(staticizerXSLFolder).exists();
        if (!staticizerXSLDir) {
            String subject = localizationres.getString("export.mail.subject.error");
            String body = localizationres.getString("export.mail.body.staticizer.folder.path").replace("{user}", dataprocedure.getUsers().getEmail());
            reportMgr.setMailSubject(subject);
            reportMgr.setMailBody(body);
            reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("The path 'staticizer.formats.path' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.error("=================The path 'staticizer.formats.path' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.==============");
            throw new Exception("The path 'staticizer.formats.path' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting. ");
        }
    }

    private void startStaticization() throws Exception {
        DataprocedureMgr dataproceduteMgr = new DataprocedureMgr(em);
        Users user = (Users) request.getSession().getAttribute(eu.europa.ec.re3gistrycommon.constants.Constants.USER_KEY);

        try {
            setWritingFilesStatus(dataproceduteMgr);
            staticizeItems();

            String subject = localizationres.getString("export.mail.subject.ok");
            String body = localizationres.getString("export.mail.body.success").replace("{user}", user.getEmail());
            reportMgr.setMailSubject(subject);
            reportMgr.setMailBody(body);
            reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_COMPLETED_STATUS);
        } catch (Exception ex) {
            String subject = localizationres.getString("staticizer.mail.subject.error");
            String body = localizationres.getString("staticizer.mail.body.failed").replace("{user}", user.getEmail());

            reportMgr.setMailSubject(subject);
            reportMgr.setMailBody(body);
            reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Has been encountered a problem while writing the files. Error message: " + ex.getMessage(), subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.error("=================Has been encountered a problem while writing the files.==============. Error message: " + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the files. Error message: " + ex.getMessage());
        }
    }

    private void setWritingFilesStatus(DataprocedureMgr dataprocedureMgr) throws Exception {
        em.getTransaction().begin();
        dataprocedureMgr.updateDataProcedureStatus(em, dataprocedure, eu.europa.ec.re3gistrycommon.constants.Constants.ID_WRITING_FILES_STATUS);
        em.getTransaction().commit();
    }

    private void staticizeItems() throws Exception {
        ChangelogMgr changelogMgr = new ChangelogMgr(em);

        /**
         * get items to be staticize with dataprocedure
         */
        List<Changelog> changelogItems = changelogMgr.getChangelogByDataProcedure(dataprocedure);

        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
        List<Languagecode> allLanguagecode = languagecodeMgr.findAll();

        LinkedHashMap<Item, List<Languagecode>> itemsTostaticize = new LinkedHashMap<Item, List<Languagecode>>();
        LinkedHashMap<Itemclass, List<Languagecode>> registerToStaticize = new LinkedHashMap<Itemclass, List<Languagecode>>();

        for (Changelog changelog : changelogItems) {
            Item item = changelog.getItem();

            if (!itemsTostaticize.containsKey(item)) {
                if (item.getVersionnumber() == 0) {
                    itemsTostaticize.put(item, allLanguagecode);
                }
            }
            Itemclass itemclass = item.getItemclass();
            if (itemclass.getParent() == null) {
                if (!registerToStaticize.containsKey(itemclass)) {
                    registerToStaticize.put(itemclass, allLanguagecode);
                }
            }
        }

        /**
         * add to the list all the folder that are created
         */
        LinkedHashMap<File, String> linkedHashMapVAR = new LinkedHashMap();

        log.info("============================================== Writing Files ==================================================");

        File standardFolder = new File(staticizerMainfolderName);

        if (!standardFolder.exists()) {
            boolean makeStandardFolder = standardFolder.mkdir();
            if (!makeStandardFolder) {
                String subject = localizationres.getString("export.mail.subject.error");
                String body = localizationres.getString("export.mail.body.staticizer.folder.path").replace("{user}", dataprocedure.getUsers().getEmail());
                reportMgr.setMailSubject(subject);
                reportMgr.setMailBody(body);
                reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("The path 'staticizer.custom.folder.path' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.error("===================The path 'staticizer.custom.folder.path' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.====================");
                throw new Exception("The path 'staticizer.custom.folder.path' from the 'Staticizer.properties' file doesn't exist. Please check and restart the procedure of exporting.");
            }
        }

        try {
            log.info("====================================== Standard && Custom Files ==================================================");
            createStandardFiles = new CreateStandardFiles(this, itemsTostaticize, linkedHashMapVAR, registerToStaticize, em);
            createStandardFiles.createFiles();
            log.info("=================================END writing Standard && Custom Files=============================================");
        } catch (Exception ex) {
            String subject = localizationres.getString("export.mail.subject.error");
            String body = localizationres.getString("export.mail.body.error.creation.standard.files").replace("{user}", dataprocedure.getUsers().getEmail());
            reportMgr.setMailSubject(subject);
            reportMgr.setMailBody(body);
            reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Files. Error message: " + ex.getMessage(), subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Files=============================================. Error message: " + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom files. Error message: " + ex.getMessage());
        }

        try {
            log.info("================ Custom XSL Files (about,extensibility,help,rss,search,searchresult,status,system) ===============");
            new CreateXSLCustomFiles(this, linkedHashMapVAR, em);
            log.info("====================END writing Custom XSL Files (about,extensibility,help,rss,search,searchresult,status,system)============");
        } catch (Exception ex) {
            String subject = localizationres.getString("export.mail.subject.error");
            String body = localizationres.getString("export.mail.body.error.creation.custom.xsl.files").replace("{user}", dataprocedure.getUsers().getEmail());
            reportMgr.setMailSubject(subject);
            reportMgr.setMailBody(body);
            reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Has been encountered a problem while writing the custom (about,extensibility,help,rss,search,searchresult,status,system) files. Error message: " + ex.getMessage(), subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Has been encountered a problem while writing the custom (about,extensibility,help,rss,search,searchresult,status,system) files.=============================================. Error message: " + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom (about,extensibility,help,rss,search,searchresult,status,system) files. Error message: " + ex.getMessage());
        }

        try {
            log.info("================================================== VAR Files ==========================================================");
            new CreateVARFiles(linkedHashMapVAR, em);
            log.info("=============================================END writing VAR files====================================================");
        } catch (Exception ex) {
            String subject = localizationres.getString("export.mail.subject.error");
            String body = localizationres.getString("export.mail.body.error.creation.var.files").replace("{user}", dataprocedure.getUsers().getEmail());
            reportMgr.setMailSubject(subject);
            reportMgr.setMailBody(body);
            reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing VAR Files. Error message: " + ex.getMessage(), subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing VAR files=============================================. Error message: " + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the VAR files. Error message: " + ex.getMessage());
        }

        try {
            log.info("================================================ SOLR Files ======================================================");
            new CreateSOLRFiles(this, itemsTostaticize, registerToStaticize, em);
            log.info("=================================================END writing SOLR Files===========================================");
        } catch (Exception ex) {
            String subject = localizationres.getString("export.mail.subject.error");
            String body = localizationres.getString("export.mail.body.error.creation.solr.files").replace("{user}", dataprocedure.getUsers().getEmail());
            reportMgr.setMailSubject(subject);
            reportMgr.setMailBody(body);
            reportMgr.setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing SOLR Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing SOLR files=============================================. Error message: " + ex.getMessage() + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the SOLR files. Error message: " + ex.getMessage());
        }

        log.info("==============================================END Writing Files ==================================================");

    }

    public String getFolderSeparator() {
        return folderSeparator;
    }

    public String getStaticizerMainfolderName() {
        return staticizerMainfolderName;
    }

    public String getStaticizerXSLFolder() {
        return staticizerXSLFolder;
    }

    public CreateStandardFiles getCreateStandardFiles() {
        return createStandardFiles;
    }

    public ReportMgr getReportMgr() {
        return reportMgr;
    }

    public ResourceBundle getLocalizationres() {
        return localizationres;
    }

    public Dataprocedure getDataprocedure() {
        return dataprocedure;
    }

}
